package com.PFE.ServerManager;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.io.*;
import java.sql.Timestamp;
import java.util.*;

import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.Authentication;
import org.yaml.snakeyaml.Yaml;
import org.yaml.snakeyaml.constructor.Constructor;

import javax.servlet.annotation.MultipartConfig;

@Controller
@MultipartConfig(fileSizeThreshold = 20971520) //à changer, taille max des fichiers

public class MainController {

    @Autowired
    NodeRepository nodeRepository;

    @Autowired
    CustomerRepository customerRepository;

    @Autowired
    TeamRepository teamRepository;

    @Autowired
    RoleRepository roleRepository;

    @Autowired
    UpdateRepository updateRepository;

    @Autowired
    BCryptPasswordEncoder bCryptPasswordEncoder;

    @GetMapping(value="/")
    public String uploadRedirection(){
        return "redirect:home";
    }

    @GetMapping(value="/home")
    public ModelAndView home() {
        ModelAndView modelAndView = new ModelAndView();
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        modelAndView.addObject("customerName", customer.getEmail().split("@")[0]);
        modelAndView.addObject("customerRole", customer.getRole());
        modelAndView.setViewName("home");
        return modelAndView;
    }

    @GetMapping(value="/upload")
    public ModelAndView upload() {
        ModelAndView modelAndView = new ModelAndView();
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        modelAndView.addObject("customerName", customer.getEmail().split("@")[0]);
        modelAndView.addObject("customerRole", customer.getRole());
        modelAndView.setViewName("upload");
        return modelAndView;
    }

    @GetMapping(value="/update")
    public ModelAndView update() {
        ModelAndView modelAndView = new ModelAndView();
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        modelAndView.addObject("customerName", customer.getEmail().split("@")[0]);
        modelAndView.addObject("customerRole", customer.getRole());
        modelAndView.addObject("customerMaj", (teamRepository.findByCustomersContaining(customer)).getUpdates());
        List<Node> nodes = nodeRepository.findAll();
        modelAndView.addObject("nodes", nodes);

        File file = new File("files");
        File[] dirs = file.listFiles();
        List<String> filesName = new ArrayList<String>();

        if (dirs != null) {
            for (int i = 0; i < dirs.length; i++) {
                if (dirs[i].isDirectory() == true) {
                    if ((dirs[i].getName().split("_")[0]).equals(customer.getEmail().split("@")[0])) {
                        File dir = new File(dirs[i].getAbsolutePath());
                        File[] f = dir.listFiles();
                        filesName.add(f[0].getName());
                   }
                }
            }
        }

        modelAndView.addObject("customerFiles", filesName);
        modelAndView.setViewName("update");
        return modelAndView;
    }

    @GetMapping(path="/registration")
    public ModelAndView registration() {
        ModelAndView modelAndView = new ModelAndView();
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        modelAndView.addObject("customerName", customer.getEmail().split("@")[0]);
        modelAndView.addObject("customerRole", customer.getRole());
        modelAndView.setViewName("registration");
        return modelAndView;
    }

    @GetMapping(path="/denied")
    public String denied() {
        return "denied";
    }

    @PostMapping(path="/registration")
    public ModelAndView addNewUser(@RequestParam String email, @RequestParam String password, @RequestParam String role) {
        //Model map, ModelAndView ou l'utilisation direct comme dans la méthode précédente sont 3 méthodes qui permettent d'envoyer des informations et donc de changer l'apparence d'une page
        ModelAndView modelAndView = new ModelAndView(); // il n'est peut être pas utile d'utiliser ModelAndView
        Customer n = new Customer();
        n.setEmail(email);
        n.setPassword(bCryptPasswordEncoder.encode(password));
        n.setCustomerId((int)(customerRepository.count() + 1));
        n.setActive(1);
        Customer temp = customerRepository.findByEmail(email);
        Role userRole = roleRepository.findByRole(role);
        n.addRole(userRole);

        //utilisé uniquement pour continuer à afficher l'utilisateur connecté//
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        modelAndView.addObject("customerName", customer.getEmail().split("@")[0]);
        modelAndView.addObject("customerRole", customer.getRole());
        modelAndView.setViewName("registration");

        List<Customer> list = customerRepository.findAll(); // attention : la méthode findAll() de JpaRepository retourne une liste alors que celle de CrudRepository retourne un itérable
        modelAndView.addObject("list", list);

        if(temp != null) {
            modelAndView.addObject("message", "L'utilisateur existe déjà !");
            modelAndView.addObject("fail", true);
        }
        else {
            modelAndView.addObject("message", "L'utilisateur a bien été ajouté !");
            modelAndView.addObject("ok", true);
            customerRepository.save(n);
        }
        modelAndView.setViewName("registration");
        return modelAndView;
    }

    @RequestMapping(value = "/file", method = RequestMethod.POST)
    @ResponseStatus(value = HttpStatus.OK)
    public void submit(@RequestParam MultipartFile file) {

        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        Timestamp timestamp = new Timestamp(System.currentTimeMillis());
        File dirs = new File("files/" + customer.getEmail().split("@")[0] + "_" + timestamp.getTime());
        dirs.mkdirs();
        OutputStream outputStream = null;
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
            File newFile = new File(dirs.getPath() + "/" + file.getOriginalFilename());
            if (!newFile.exists()) {
                newFile.createNewFile();
            }
            outputStream = new FileOutputStream(newFile);
            int read = 0;
            byte[] bytes = new byte[1024];

            while((read = inputStream.read(bytes)) != -1) {
                outputStream.write(bytes, 0, read);
            }
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        finally {
            try {
                outputStream.close();
            }
            catch(IOException e) {
            }
        }
    }

    @RequestMapping(value = "/config", method = RequestMethod.POST)
    @ResponseStatus(value = HttpStatus.OK)
    public void submitConfig(@RequestParam MultipartFile file) {

        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        nodeRepository.deleteAll();
        Yaml yaml = new Yaml(new Constructor(ConfigNodes.class));
        ConfigNodes conf = yaml.load(inputStream);

        Iterator<NodeYAML> iter = conf.getNodes().iterator();
        while (iter.hasNext()) {
            NodeYAML element = iter.next();
            Node node = new Node();
            node.setName(element.getName());
            node.setArch(element.getArch());
            node.setIp(element.getIp());
            nodeRepository.save(node);
        }
    }

    @GetMapping(path="/login")
    public ModelAndView login() {
        ModelAndView modelAndView = new ModelAndView();
        modelAndView.setViewName("login");
        return modelAndView;
    }

    @GetMapping(path="/all")
    public ModelAndView getAllUsers() {
        ModelAndView modelAndView = new ModelAndView();
        modelAndView.setViewName("all");

        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        modelAndView.addObject("customerName", customer.getEmail().split("@")[0]);
        modelAndView.addObject("customerRole", customer.getRole());

        List<Customer> list = customerRepository.findAll(); // attention : la méthode findAll() de JpaRepository retourne une liste alors que celle de CrudRepository retourne un itérable
        modelAndView.addObject("list", list);
        return modelAndView;
    }

    @PostMapping(path="/savemaj")
    public String saveMaj(@RequestParam String name, @RequestParam String date, @RequestParam String nodes, @RequestParam String file){
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());

        Update update_c = new Update();
        update_c.setUpdate(name);
        update_c.setDate(date);
        update_c.setNodes(nodes);
        update_c.setFile(file);

        updateRepository.save(update_c); // ajouter la mise a jour dans la table

        System.out.println("team name : " + (teamRepository.findByCustomersContaining(customer)).getTeam());
        Team teamOfCustomer = teamRepository.findByCustomersContaining(customer);
        teamOfCustomer.addUpdate(update_c);
        teamRepository.save(teamOfCustomer); // permet de rendre effective la jointure entre customer et maj

        return "redirect:/update";
    }

    @PostMapping(path="/runmaj")
    public String runMaj(@RequestParam String name, @RequestParam String date, @RequestParam String nodes, @RequestParam String file){
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());

        Update update_c = new Update();
        update_c.setUpdate(name);
        update_c.setDate(date);
        update_c.setNodes(nodes);
        update_c.setFile(file);

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("name", update_c.getUpdate());
        data.put("date", update_c.getDate());
        data.put("file", update_c.getFile());
        data.put("nodes", update_c.getNodes().split(";"));
        Yaml yaml = new Yaml();
        FileWriter writer = null;
        try {
            writer = new FileWriter("toflash/" + customer.getEmail().split("@")[0] + "_" + update_c.getUpdate() + ".yaml");
        } catch (IOException e) {
            e.printStackTrace();
        }
        yaml.dump(data, writer);

        return "redirect:/update";
    }

    @PostMapping(path="/startsavedmaj")
    @ResponseStatus(value = HttpStatus.OK)
    public void startSavedMaj(@RequestParam String majname){
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = customerRepository.findByEmail(auth.getName());
        Update update = updateRepository.findByUpdate(majname);
        Map<String, Object> data = new HashMap<String, Object>();
        data.put("name", update.getUpdate());
        data.put("date", update.getDate());
        data.put("file", update.getFile());
        data.put("nodes", update.getNodes().split(";"));
        Yaml yaml = new Yaml();
        FileWriter writer = null;
        try {
            writer = new FileWriter("toflash/" + customer.getEmail().split("@")[0] + "_" + update.getUpdate() + ".yaml");
        } catch (IOException e) {
            e.printStackTrace();
        }
        yaml.dump(data, writer);
    }
}